@@ -4,16 +4,18 @@ from django.conf.urls import include, patterns, url
 from .views import IndexView
 from .views import UserMessageView, UserMessagesView, SendMessageView
 from .views import PublicProfileView, ProfileView, EditProfileView
-from .views import CategoryView, ForumView
+from .views import CategoryView, ForumView, RegisterProfileView
+from .views import ThreadView, NewThreadView
 
 urlpatterns = patterns('',
     url(r'^$', IndexView.as_view(), name='index'),
 
     # your own profile
-    url(r'^profile/$', ProfileView.as_view(), name='profile'),
+    url(r'^profile/$', ProfileView.as_view(),
+        name='profile'),
 
-    url(r'^profile/register/',
-        'forum.views.register_profile_view', name='register_profile'),
+    url(r'^profile/register/', RegisterProfileView.as_view(),
+        name='register_profile'),
 
     # edit your own profile
     url(r'^profile/edit/$', EditProfileView.as_view(), name='edit_profile'),
@@ -41,11 +43,11 @@ urlpatterns = patterns('',
 
     # post a new thread
     url(r'^(?P<pk_forum>\d+)/threads/new/$',
-        'forum.views.new_thread_view', name='new_thread'),
+        NewThreadView.as_view(), name='new_thread'),
 
     # read a thread
     url(r'^(?P<pk_forum>\d+)/threads/(?P<pk_thread>\d+)/$',
-        'forum.views.thread_view', name='thread'),
+        ThreadView.as_view(), name='thread'),
 #
 #    # edit a thread
 #    (r'^(?P<pk_forum>\d+)/threads/(?P<pk_thread>\d+)/edit/$',
