@@ -6,7 +6,7 @@ import json
 from django.core.urlresolvers import reverse
 from django.db import transaction
 from django.core.paginator import Paginator
-from django.http import HttpResponse
+from django.http import HttpResponse, HttpResponseForbidden
 from django.shortcuts import get_object_or_404 as get_object
 from django.shortcuts import render, redirect
 from django.utils.translation import ugettext_lazy as _
@@ -16,6 +16,7 @@ from django.utils.decorators import available_attrs
 from django.utils.decorators import method_decorator
 from django.views.generic import TemplateView, ListView, DetailView
 from django.views.generic.edit import FormView, CreateView, UpdateView
+from django.views.generic.edit import FormMixin
 
 from app_settings import *
 from .constants import MARKUP_EDITORS
@@ -219,64 +220,97 @@ class ForumView(TemplateView):
         return context
 
 
-class ThreadView(TemplateView):
+class ThreadView(ListView):
+    pk_url_kwarg = 'pk_thread'
     template_name = 'forum/thread.html'
+    paginate_by = PAGINATE_REPLIES_BY
+
+    def dispatch(self, request, *args, **kwargs):
+        self.forum = get_object(Forum.objects.active(), pk=kwargs['pk_forum'])
+        self.category = self.forum.category
+        self.thread = self.get_object(self.forum.threads.active())
+        return super(ThreadView, self).dispatch(request, *args, **kwargs)
+
+    def get_object(self, queryset):
+        pk = self.kwargs.get(self.pk_url_kwarg, None)
+        return get_object(queryset, pk=pk)
+
+    def get_queryset(self):
+        return self.thread.replies.active()
+
+    def get_context_data(self, **kwargs):
+        context = super(ThreadView, self).get_context_data(**kwargs)
+        context['forum'] = self.forum
+        context['category'] = self.category
+        context['thread'] = self.thread
+        return context
+
+    def get(self, request, *args, **kwargs):
+        self.thread.update_views(request)
+        return super(ThreadView, self).get(request, *args, **kwargs)
 
 
-def thread_view(request, pk_forum, pk_thread):
-    forum = get_object(Forum.objects.active(), pk=pk_forum)
-    category = forum.category
-    threads = forum.threads.active()
-    thread = get_object(threads, pk=pk_thread)
-    replies = thread.replies.active()
-    page_number = request.GET.get('page', 1)
-    object_list = Paginator(replies, PAGINATE_REPLIES_BY).page(page_number)
-    form = None
+class NewThreadView(ProtectedMixin, CreateView):
+    form_classes = [ThreadForm, ReplyForm]
+    context_object_names = ['thread_form', 'reply_form']
+    template_name = 'forum/new_thread.html'
 
-    # FIXME - form reference problem
-    if ALLOW_INLINE_REPLY:
-        form = ReplyForm(request.POST or None)
+    @transaction.atomic
+    def dispatch(self, *args, **kwargs):
+        self.forum = get_object(
+            Forum.objects.active(),
+            pk=kwargs['pk_forum'])
+        self.category = self.forum.category
+        self.object = None
 
-        if form.is_bound and form.is_valid():
-            new_reply = form.save()
-            return redirect(new_reply)
+        return super(NewThreadView, self).dispatch(*args, **kwargs)
 
-    thread.update_views(request)
-    return render(request, 'forum/thread.html', {
-        'category': category,
-        'form': form,
-        'forum': forum,
-        'thread': thread,
-        'object_list': object_list
-    })
+    def get_context_data(self, **kwargs):
+        context = super(NewThreadView, self).get_context_data(**kwargs)
+        context['forum'] = self.forum
+        context['category'] = self.category
+        return context
 
+    def get_form_class_list(self):
+        return self.form_classes
 
-@login_required
-@profile_required
-@transaction.atomic
-def new_thread_view(request, pk_forum, pk_thread=None):
-    forum = get_object(Forum.objects.active(), pk=pk_forum)
-    category = forum.category
+    def get_form_list(self, form_classes):
+        return [form_class(**self.get_form_kwargs())
+                for form_class in form_classes]
 
-    thread_form = ThreadForm(
-        request.POST or None,
-        request.method == 'POST' and request.FILES or None)
-    reply_form = ReplyForm(
-        request.POST or None,
-        request.method == 'POST' and request.FILES or None)
-    profile = request.user.forum_profile
+    def get(self, request, *args, **kwargs):
+        """
+        Handles GET requests and instantiates a blank version of the form.
+        """
+        form_class_list = self.get_form_class_list()
+        form_list = self.get_form_list(form_class_list)
+        context = self.get_context_data()
+        for i in range(len(self.context_object_names)):
+            name = self.context_object_names[i]
+            context[name] = form_list[i]
+        return self.render_to_response(context)
 
-    if thread_form.is_bound and reply_form.is_bound \
-            and thread_form.is_valid() and reply_form.is_valid():
-        new_thread = thread_form.save(profile, forum)
-        reply_form.save(profile, forum, new_thread)
+    def post(self, request, *args, **kwargs):
+        """
+        Handles POST requests, instantiating a form instance with the passed
+        POST variables and then checked for validity.
+        """
+        form_class_list = self.get_form_class_list()
+        form_list = self.get_form_list(form_class_list)
+        if all([form.is_valid() for form in form_list]):
+            return self.form_valid(form_list)
+        else:
+            return self.form_invalid(form_list)
+
+    def form_valid(self, form_list):
+        new_thread = None
+
+        for form in form_list:
+            if isinstance(form, ThreadForm):
+                new_thread = form.save(self.profile, self.forum)
+            if isinstance(form, ReplyForm):
+                form.save(self.profile, self.forum, new_thread)
         return redirect(new_thread)
-    return render(request, 'forum/new_thread.html', {
-        'category': category,
-        'forum': forum,
-        'thread_form': thread_form,
-        'reply_form': reply_form
-    })
 
 
 @login_required
@@ -327,19 +361,21 @@ def rate_reply_view(request, pk_forum, pk_thread, pk_reply):
         mimetype='application/json')
 
 
-@login_required
-def register_profile_view(request):
-    next = request.REQUEST.get('next', None)
+class RegisterProfileView(TemplateView):
+    template_name = 'forum/register_profile.html'
 
-    if request.method == 'POST':
+    @method_decorator(login_required)
+    def dispatch(self, *args, **kwargs):
+        return super(RegisterProfileView, self).dispatch(*args, **kwargs)
+
+    def post(self, request, *args, **kwargs):
+        next = request.REQUEST.get('next', None)
         profile, created = Profile.objects.get_or_create(user=request.user)
+        # TODO: make this a event
         box = MessageBox.objects.get_or_create(profile=profile)
         messages.info(request, _("You now have a forum profile! Please, enjoy ; )"))
 
         if next is not None and is_path(next):
             return redirect(next)
 
-        return redirect('forum:index')
-
-    return render(request, 'forum/register_profile.html',
-                  {'next': next})
\ No newline at end of file
+        return redirect('forum:index')
\ No newline at end of file
